import { selectSource } from './common';

const TARGET_TYPES = [STRUCTURE_EXTENSION, STRUCTURE_SPAWN, STRUCTURE_TOWER];

interface HarvesterMemory extends CreepMemory {
  sourceId: Id<Source> | undefined;
}

export function runAsHarvester(creep: Creep) {
  const memory = creep.memory as HarvesterMemory;

  if (memory.sourceId === undefined) {
    memory.sourceId = selectSource(creep);
  }

  if (creep.store.getFreeCapacity() > 0) {
    const source = Game.getObjectById(memory.sourceId);
    if (source && creep.harvest(source) === ERR_NOT_IN_RANGE) {
      creep.say('🔄 harvest');
      creep.moveTo(source, { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  } else {
    const target = findTarget(creep);
    if (target) {
      moveEnergyToTarget(creep, target);
    } else {
      moveToSpawn(creep);
    }
  }
}

function findTarget(creep: Creep): Structure | undefined {
  for (let type of TARGET_TYPES) {
    const targets = creep.room.find(FIND_STRUCTURES, {
      filter: (structure) => {
        return structure.structureType === type && structure.store.getFreeCapacity(RESOURCE_ENERGY) > 0;
      }
    });
    if (targets.length) {
      return targets[0];
    }
  }
  return undefined;
}

function moveEnergyToTarget(creep: Creep, target: Structure) {
  if (creep.transfer(target, RESOURCE_ENERGY) === ERR_NOT_IN_RANGE) {
    creep.say('🚛 transfer');
    creep.moveTo(target, { visualizePathStyle: { stroke: '#ffffff' } });
  }
}

function moveToSpawn(creep: Creep) {
  const spawns = creep.room.find(FIND_MY_SPAWNS);
  if (spawns.length > 0) {
    creep.say('to spawn');
    creep.moveTo(spawns[0]);
  }
}
